import os
import sys
import shutil
import re
import subprocess
import glob

###############################################################################
# Settings are in file settings.py
###############################################################################
from settings import *

###############################################################################
# Don't edit below
###############################################################################

def main():
	longest_sequence = sorted(sequence_lengths)[-1]

	if len(sys.argv) < 2:
		sys.stderr.write('Usage:\n')
		sys.stderr.write(' * python %s all\n' % sys.argv[0])
		sys.stderr.write('   Runs all sequence lengths and all sets\n')
		sys.stderr.write(' * python %s 10\n' % sys.argv[0])
		sys.stderr.write('   Runs all sets for sequence length 10\n')
		sys.stderr.write(' * python %s 4-8 0,3\n' % sys.argv[0])
		sys.stderr.write('   Runs sets 0 and 3 for sequence lengths 4 to 6\n')
		sys.exit(1)
	
	if not os.path.isdir(root_path):
		sys.stderr.write('Root folder does not exist: %s\nEdit settings.py!\n' % root_path)
		sys.exit(1)
	
	if not os.path.exists(superres_binary):
		sys.stderr.write('superres binary not found at: %s\nEdit settings.py!\n' % superres_binary)
		sys.exit(1)
	
	run_seq_lengths = []
	run_sets = []

	# Parse sequence lengths

	if sys.argv[1] == 'all':
		run_seq_lengths = sequence_lengths
		run_sets        = range(max_sets)

	else:
		run_seq_lengths = parse_number_range(sys.argv[1], sequence_lengths)

	if len(run_seq_lengths) == 0:
		sys.stderr.write('Couldn\'t parse sequence length argument: "%s"' % sys.argv[1])
		sys.exit(1)
	
	# Parse sets

	if sys.argv[1] != 'all':
		if len(sys.argv) < 3:
			run_sets = range(max_sets)
		else:
			run_sets = parse_number_range(sys.argv[2], range(max_sets))
	
	if len(run_sets) == 0:
		sys.stderr.write('Couldn\'t parse set argument: "%s"' % sys.argv[2])
		sys.exit(1)

	# Run

	for sl in run_seq_lengths:
		for s in run_sets:
			for scale in [1, 2, 3, 4, 5]:
				run(sl, s, scale)

###############################################################################
# Helpers
###############################################################################

def run(sl, s, scale):
	print 'Running sequence length %d, set %d, scale %d...' % (sl, s, scale),
	sys.stdout.flush()
	
	sl_folder = 'sequencelength%02d' % sl
	s_folder  = 'set%02d' % s
	
	set_path = os.path.join(os.path.join(root_path, sl_folder), s_folder)
	
	if not os.path.isdir(set_path):
		sys.stderr.write('Couldn\'t find set folder at: %s' % set_path)
		sys.exit(1)
	
	images_path = os.path.join(set_path, image_filename)
	motion_path = os.path.join(set_path, motion_filename)
	
	if not os.path.exists(motion_path):
		sys.stderr.write('Couldn\'t find set motion file at: %s' % motion_path)
		sys.exit(1)
		
	# Remove old output
	
	wh = 200 * scale
	
	old_files = glob.glob(os.path.join(set_path, '%d_%d_*' % (wh, wh)))
	for f in old_files:
		os.remove(f)
		
	if show_superres_output:
		out_out = sys.stdout
		out_err = sys.stderr
	else:
		out_out = open(os.devnull, 'w')
		out_err = out_out
	
	extension = 'png'
	if image_path[-3:] == 'txt':
		extension = 'txt'
	
	for i in xrange(superres_num_executions):
		subprocess.call([
			superres_binary, 
			images_path, 
			motion_path, 
			'high_%02d_%02d_%02d.%s' % (sl, s, scale, extension),
			str(wh), str(wh), 
			'0.4',
			str(superres_iterations)
			],
			cwd=set_path,
			stdout=out_out,
			stderr=out_err
		)
	
	if not show_superres_output:
		out_out.close()
		
	# Compute avg. time
	
	with open(os.path.join(set_path, '%d_%d_0.4_high_%02d_%02d_%02d.%s_superres.txt' % (wh, wh, sl, s, scale, extension))) as f:
		times = f.readlines()
		timesum = sum(map(float, times))
		avgtime = timesum / float(superres_num_executions)
		with open(os.path.join(set_path, 'avg_time_scale_%02d.txt' % scale), 'w') as af:
			af.write('%f\n' % avgtime)
	
	print('avg. time: %f' % avgtime)

def parse_number_range(theinput, possible_values):
	if '-' in theinput:
		r = re.match('(\d+)\-(\d+)', theinput)
	
		if not r:
			sys.stderr.write('Couldn\'t parse argument: "%s"' % theinput)
			sys.exit(1)
	
		sl_start = int(r.group(1))
		sl_end   = int(r.group(2))
	
		therange = [x for x in range(sl_start, sl_end+1) if x in possible_values]
	
	elif ',' in theinput:
		therange = []
	
		for x in theinput.split(','):
			if x.isdigit() and int(x) in possible_values:
				therange += [int(x)]

	elif theinput.isdigit() and int(theinput) in possible_values:
		therange = [int(theinput)]

	else:
		sys.stderr.write('Couldn\'t parse argument: "%s"' % theinput)
		sys.exit(1)
		
	return therange

###############################################################################
# Run main
###############################################################################

if __name__ == '__main__':
	main()
